@@ -442,7 +442,7 @@ class DatalinkQuery(DALQuery):
         DALFormatError
            for errors parsing the VOTable response
         """
-        return DatalinkResults(self.execute_votable(post),
+        return DatalinkResults(self.execute_votable(post=post),
                                url=self.queryurl, session=self._session)
 
 
@@ -467,7 +467,7 @@ class DatalinkResults(DatalinkResultsMixin, DALResults):
 
     ``DatalinkResults`` is essentially a wrapper around an Astropy
     :py:mod:`~astropy.io.votable`
-    :py:class:`~astropy.io.votable.tree.Table` instance where the
+    :py:class:`~astropy.io.votable.tree.TableElement` instance where the
     columns contain the various metadata describing the images.
     One can access that VOTable directly via the
     :py:attr:`~pyvo.dal.query.DALResults.votable` attribute.  Thus,
@@ -822,7 +822,7 @@ class DatalinkRecord(DatalinkRecordMixin, SodaRecordMixin, Record):
 class AxisParamMixin():
     """
     Stores the axis parameters (pos, band, time and pol) used in SODA
-    or SIAv2 queries
+    or SIA2 queries
     """
     @property
     def pos(self):
