@@ -32,7 +32,7 @@ import collections
 
 from warnings import warn
 
-from astropy.table import Table
+from astropy.table import Table, QTable
 from astropy.io.votable import parse as votableparse
 from astropy.io.votable.ucd import parse_ucd
 from astropy.utils.exceptions import AstropyDeprecationWarning
@@ -165,7 +165,7 @@ class DALQuery(dict):
         DALFormatError
            for errors parsing the VOTable response
         """
-        return DALResults(self.execute_votable(), self.queryurl, session=self._session)
+        return DALResults(self.execute_votable(), url=self.queryurl, session=self._session)
 
     def execute_raw(self):
         """
@@ -223,8 +223,8 @@ class DALQuery(dict):
 
         Returns
         -------
-        astropy.io.votable.tree.Table
-           an Astropy votable Table instance
+        astropy.io.votable.tree.VOTableFile
+           an Astropy votable object
 
         Raises
         ------
@@ -401,7 +401,7 @@ class DALResults:
         return infos
 
     def __repr__(self):
-        return repr(self.to_table())
+        return f"<DALResults{repr(self.to_table())[1:]}"
 
     @property
     def queryurl(self):
@@ -420,7 +420,7 @@ class DALResults:
     @property
     def resultstable(self):
         """
-        The votable XML element `astropy.io.votable.tree.Table`
+        The votable XML element `astropy.io.votable.tree.TableElement`
         """
         return self._resultstable
 
@@ -434,6 +434,17 @@ class DALResults:
         """
         return self.resultstable.to_table(use_names_over_ids=True)
 
+    def to_qtable(self):
+        """
+        Returns a astropy QTable object containing quantities instead of simple
+        values
+
+        Returns
+        -------
+        `astropy.table.QTable`
+        """
+        return QTable(self.resultstable.to_table(use_names_over_ids=True))
+
     @property
     def table(self):
         warn(AstropyDeprecationWarning(
@@ -822,7 +833,7 @@ class Record(Mapping):
             bufsize = 524288
 
         if not filename:
-            filename = self.make_dataset_filename(dir)
+            filename = self.make_dataset_filename(dir=dir)
 
         inp = self.getdataset(timeout)
         try:
@@ -867,7 +878,7 @@ class Record(Mapping):
         if not base:
             base = self.suggest_dataset_basename()
         if not ext:
-            ext = self.suggest_extension("dat")
+            ext = self.suggest_extension(default="dat")
 
         # be efficient when writing a bunch of files into the same directory
         # in succession
