@@ -113,7 +113,8 @@ def search(
     pyvo.dal.query.DALQueryError
     """
     service = SIAService(url)
-    return service.search(pos, size, format, intersect, verbosity, **keywords)
+    return service.search(pos=pos, size=size, format=format, intersect=intersect,
+                          verbosity=verbosity, **keywords)
 
 
 class SIAService(DALService):
@@ -255,7 +256,7 @@ class SIAService(DALService):
         pyvo.dal.query.DALQueryError
         """
         return self.create_query(
-            pos, size, format, intersect, verbosity, **keywords).execute()
+            pos=pos, size=size, format=format, intersect=intersect, verbosity=verbosity, **keywords).execute()
 
     def create_query(
             self, pos=None, size=None, format=None, intersect=None,
@@ -317,7 +318,8 @@ class SIAService(DALService):
         SIAQuery
         """
         return SIAQuery(
-            self.baseurl, pos, size, format, intersect, verbosity, self._session, **keywords)
+            self.baseurl, pos=pos, size=size, format=format, intersect=intersect,
+            verbosity=verbosity, session=self._session, **keywords)
 
     def describe(self):
         print(self.description)
@@ -594,7 +596,7 @@ class SIAResults(DatalinkResultsMixin, DALResults):
 
     ``SIAResults`` is essentially a wrapper around an Astropy
     :py:mod:`~astropy.io.votable`
-    :py:class:`~astropy.io.votable.tree.Table` instance where the
+    :py:class:`~astropy.io.votable.tree.TableElement` instance where the
     columns contain the various metadata describing the images.
     One can access that VOTable directly via the
     :py:attr:`~pyvo.dal.query.DALResults.votable` attribute.  Thus,
