@@ -5,7 +5,6 @@ A module for accessing remote source and observation catalogs
 from functools import partial
 from datetime import datetime
 from time import sleep
-from packaging.version import Version
 
 import requests
 from urllib.parse import urlparse, urljoin
@@ -97,7 +96,7 @@ def search(url, query, language="ADQL", maxrec=None, uploads=None, **keywords):
         an error, including a query syntax error.
     """
     service = TAPService(url)
-    return service.search(query, language, maxrec, uploads, **keywords)
+    return service.search(query, language=language, maxrec=maxrec, uploads=uploads, **keywords)
 
 
 class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
@@ -128,6 +127,20 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         if hasattr(self._session, 'update_from_capabilities'):
             self._session.update_from_capabilities(self.capabilities)
 
+    def get_tap_capability(self):
+        """
+        returns the (first) TAP capability of this service.
+
+        Returns
+        -------
+        A `~pyvo.io.vosi.tapregext.TableAccess` instance.
+        """
+        for capa in self.capabilities:
+            if isinstance(capa, tr.TableAccess):
+                return capa
+        raise DALServiceError("Invalid TAP service: Does not"
+            " expose a tr:TableAccess capability")
+
     @property
     def tables(self):
         """
@@ -150,6 +163,38 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
                 vosi.parse_tables(response.raw.read), tables_url)
         return self._tables
 
+    def _parse_examples(self, examples_uri, *, depth=0):
+        """returns the TAP queries from a DALI examples URI.
+        """
+        if depth > 5:
+            raise DALServiceError("Suspecting endless recursion when"
+                " parsing TAP examples")
+
+        response = self._session.get(examples_uri, stream=True)
+        if response.status_code == 404:
+            return []
+
+        try:
+            response.raise_for_status()
+        except requests.RequestException as ex:
+            raise DALServiceError.from_except(ex, examples_uri)
+
+        try:
+            root = xml.etree.ElementTree.parse(
+                io.BytesIO(response.content)).getroot()
+            exampleElements = root.findall('.//*[@property="query"]')
+        except Exception as ex:
+            raise DALServiceError.from_except(ex, examples_uri)
+
+        examples = [TAPQuery(self.baseurl, example.text)
+            for example in exampleElements]
+
+        for continuation in root.findall('.//*[@property="continuation"]'):
+            examples.extend(
+                self._parse_examples(continuation.get("href"), depth=depth + 1))
+
+        return examples
+
     @property
     def examples(self):
         """
@@ -158,23 +203,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         if self._examples is None:
             examples_url = '{}/examples'.format(self.baseurl)
 
-            response = self._session.get(examples_url, stream=True)
-            if response.status_code == 404:
-                return []
-
-            try:
-                response.raise_for_status()
-            except requests.RequestException as ex:
-                raise DALServiceError.from_except(ex, examples_url)
-
-            try:
-                root = xml.etree.ElementTree.parse(io.BytesIO(response.content)).getroot()
-                exampleElements = root.findall('.//*[@property="query"]')
-            except Exception as ex:
-                raise DALServiceError.from_except(ex, examples_url)
-
-            self._examples = [TAPQuery(self.baseurl, example.text) for example in exampleElements]
-
+            self._examples = self._parse_examples(examples_url)
         return self._examples
 
     @property
@@ -188,9 +217,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             if the property is not exposed by the service
         """
         try:
-            for capa in self.capabilities:
-                if isinstance(capa, tr.TableAccess):
-                    return capa.outputlimit.default.content
+            return self.get_tap_capability().outputlimit.default.content
         except AttributeError:
             pass
         raise DALServiceError("Default limit not exposed by the service")
@@ -206,9 +233,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             if the property is not exposed by the service
         """
         try:
-            for capa in self.capabilities:
-                if isinstance(capa, tr.TableAccess):
-                    return capa.outputlimit.hard.content
+            return self.get_tap_capability().outputlimit.hard.content
         except AttributeError:
             pass
         raise DALServiceError("Hard limit not exposed by the service")
@@ -219,11 +244,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         a list of upload methods in form of
         :py:class:`~pyvo.io.vosi.tapregext.UploadMethod` objects
         """
-        upload_methods = []
-        for capa in self.capabilities:
-            if isinstance(capa, tr.TableAccess):
-                upload_methods += capa.uploadmethods
-        return upload_methods
+        return self.get_tap_capability().uploadmethods
 
     def run_sync(
             self, query, language="ADQL", maxrec=None, uploads=None,
@@ -297,7 +318,8 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         AsyncTAPJob
         """
         job = AsyncTAPJob.create(
-            self.baseurl, query, language, maxrec, uploads, self._session, **keywords)
+            self.baseurl, query, language=language, maxrec=maxrec, uploads=uploads,
+            session=self._session, **keywords)
         job = job.run().wait()
         job.raise_if_error()
         result = job.fetch_result()
@@ -334,7 +356,8 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         AsyncTAPJob
         """
         return AsyncTAPJob.create(
-            self.baseurl, query, language, maxrec, uploads, self._session, **keywords)
+            self.baseurl, query, language=language, maxrec=maxrec, uploads=uploads,
+            session=self._session, **keywords)
 
     def create_query(
             self, query=None, mode="sync", language="ADQL", maxrec=None,
@@ -362,7 +385,8 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
             a mapping from table names to objects containing a votable.
         """
         return TAPQuery(
-            self.baseurl, query, mode, language, maxrec, uploads, self._session, **keywords)
+            self.baseurl, query, mode=mode, language=language, maxrec=maxrec,
+            uploads=uploads, session=self._session, **keywords)
 
     def get_job(self, job_id):
         """
@@ -931,10 +955,9 @@ class AsyncTAPJob:
             if cur_phase in phases:
                 break
 
-            # fallback for uws 1.0
-            if Version(self._job.version) < Version("1.1"):
-                sleep(interval)
-                interval = min(120, interval * increment)
+            # fallback for uws 1.0 or unsupported WAIT parameter
+            sleep(interval)
+            interval = min(120, interval * increment)
 
         return self
 
@@ -1133,7 +1156,7 @@ class TAPResults(DatalinkResultsMixin, DALResults):
 
     ``TAPResults`` is essentially a wrapper around an Astropy
     :py:mod:`~astropy.io.votable`
-    :py:class:`~astropy.io.votable.tree.Table` instance where the
+    :py:class:`~astropy.io.votable.tree.TableElement` instance where the
     columns contain the various metadata describing the images.
     One can access that VOTable directly via the
     :py:attr:`~pyvo.dal.query.DALResults.votable` attribute.  Thus,
