@@ -22,10 +22,13 @@ import os
 import warnings
 
 from astropy import table
+from astropy.utils.decorators import deprecated
 from astropy.utils.exceptions import AstropyDeprecationWarning
 
+import numpy
+
 from . import rtcons
-from ..dal import scs, sia, ssa, sla, tap, query as dalq
+from ..dal import scs, sia, sia2, ssa, sla, tap, query as dalq
 from ..io.vosi import vodataservice
 from ..utils.formatting import para_format_desc
 
@@ -69,26 +72,82 @@ def expand_stdid(s):
     return "ivo://ivoa.net/std/" + s
 
 
+def regularize_SIA2_id(standard_id):
+    """returns standard_id with SIA2 standard ids modified to what they should
+    have been.
+
+    Regrettably, SIA2 uses the same standardID as SIA1; sure, they use
+    different fragments, but that doesn't really help with the logic
+    we have here.
+
+    To make up for that, we replace them with the sia2 ids they should
+    have had on input.  This function assumes lowercased ids as they
+    come from RegTAP services.
+    """
+    if standard_id.startswith("ivo://ivoa.net/std/sia#query-2"):
+        return "ivo://ivoa.net/std/sia2"
+    elif standard_id.startswith("ivo://ivoa.net/std/sia#query-aux-2"):
+        # query-aux-2 is mentioned in discovering data collections,
+        # which isn't really the place to define this.  But then
+        # it's endorsed, and SIA2 doesn't say anything about it.
+        return "ivo://ivoa.net/std/sia2#aux"
+    else:
+        return standard_id
+
+
 @functools.lru_cache(1)
 def get_RegTAP_service():
     """
     a lazily created TAP service offering the RegTAP services.
 
-    This uses regtap.REGISTRY_BASEURL.  Always get the TAP service
-    there using this function to avoid re-creating the server
-    and profit from caching of capabilties, tables, etc.
+    Always get the TAP service there using this function to avoid
+    re-creating the server and profit from caching of capabilties,
+    tables, etc.
+
+    To switch to a different RegTAP service, use
+    :py:func:`choose_RegTAP_service`.
     """
     return tap.TAPService(REGISTRY_BASEURL)
 
 
+def choose_RegTAP_service(access_url):
+    """
+    changes the RegTAP service used by :py:func:`search`
+    to the one at access_url.
+
+    By default, pyVO uses whatever is given in the environment variable
+    ``IVOA_REGISTRY``, defaulting to GAVO's TAP service.  In order to
+    change the service used on the fly, always use this function in order
+    to clear caches that need clearing.
+
+    Parameters
+    ----------
+    access_url : str
+        The TAP access URL of the new RegTAP endpoints.
+        To find alternate endpoints, try ``regsearch(datamodel='regtap')``
+        and look at ``.get_interface("tap").access_url`` of the results.
+    """
+    global REGISTRY_BASEURL
+    get_RegTAP_service.cache_clear()
+    REGISTRY_BASEURL = access_url
+
+
 def get_RegTAP_query(*constraints: rtcons.Constraint,
-                     includeaux=False, **kwargs):
+                     includeaux=False,
+                     service=None,
+                     **kwargs):
     """returns SQL for a RegTAP query for constraints and keywords.
 
     This function's parameters are as for search; this is basically
     a wrapper for rtcons.build_regtap_query maintaining the legacy
     keyword-based interface.
     """
+    # we don't document the service parameter -- it's probably not useful
+    # to users and is just the conscequence of having retrofitted service
+    # sensing into the API.
+    if service is None:
+        service = get_RegTAP_service()
+
     constraints = list(constraints) + rtcons.keywords_to_constraints(kwargs)
 
     # maintain legacy includeaux by locating any Servicetype constraints
@@ -98,10 +157,13 @@ def get_RegTAP_query(*constraints: rtcons.Constraint,
             if isinstance(constraint, rtcons.Servicetype):
                 constraints[index] = constraint.include_auxiliary_services()
 
-    return rtcons.build_regtap_query(constraints)
+    return rtcons.build_regtap_query(constraints, service)
 
 
-def search(*constraints: rtcons.Constraint, includeaux: bool = False, maxrec: int = None, **kwargs):
+def search(*constraints: rtcons.Constraint,
+        includeaux: bool = False,
+        maxrec: int = None,
+        **kwargs):
 
     """
     execute a simple query to the RegTAP registry.
@@ -182,7 +244,10 @@ def search(*constraints: rtcons.Constraint, includeaux: bool = False, maxrec: in
     service = get_RegTAP_service()
     query = RegistryQuery(
         service.baseurl,
-        get_RegTAP_query(*constraints, includeaux=includeaux, **kwargs),
+        get_RegTAP_query(*constraints,
+            includeaux=includeaux,
+            service=service,
+            **kwargs),
         maxrec=maxrec)
     return query.execute()
 
@@ -202,7 +267,7 @@ class RegistryQuery(tap.TAPQuery):
         DALFormatError
            for errors parsing the VOTable response
         """
-        return RegistryResults(self.execute_votable(), self.queryurl)
+        return RegistryResults(self.execute_votable(), url=self.queryurl)
 
 
 class RegistryResults(dalq.DALResults):
@@ -314,6 +379,7 @@ class Interface:
     service_for_standardid = {
         "ivo://ivoa.net/std/conesearch": scs.SCSService,
         "ivo://ivoa.net/std/sia": sia.SIAService,
+        "ivo://ivoa.net/std/sia2": sia2.SIA2Service,
         "ivo://ivoa.net/std/ssa": ssa.SSAService,
         "ivo://ivoa.net/std/sla": sla.SLAService,
         "ivo://ivoa.net/std/tap": tap.TAPService}
@@ -321,14 +387,18 @@ class Interface:
     def __init__(self, access_url, standard_id, intf_type, intf_role):
         self.access_url = access_url
         self.standard_id = standard_id or None
-        self.is_vosi = standard_id.startswith("ivo://ivoa.net/std/vosi")
         self.type = intf_type or None
         self.role = intf_role or None
         self.is_standard = self.role == "std"
 
+        if self.standard_id is not None:
+            self.is_vosi = self.standard_id.startswith("ivo://ivoa.net/std/vosi")
+        else:
+            self.is_vosi = False
+
     def __repr__(self):
-        return (f"Interface({self.access_url!r}, {self.standard_id!r},"
-                f" {self.type!r}, {self.role!r})")
+        return (f"Interface({self.access_url!r}, standard_id={self.standard_id!r},"
+                f" intf_type={self.type!r}, intf_role={self.role!r})")
 
     def to_service(self):
         if self.type == "vr:webbrowser":
@@ -344,7 +414,10 @@ class Interface:
             raise ValueError("PyVO has no support for interfaces with"
                              f" standard id {self.standard_id}.")
 
-        return service_class(self.access_url)
+        if service_class == sia2.SIA2Service:
+            return service_class(self.access_url, check_baseurl=False)
+        else:
+            return service_class(self.access_url)
 
     def supports(self, standard_id):
         """returns true if we believe the interface should be able to talk
@@ -365,6 +438,7 @@ class Interface:
         """
         if not self.standard_id:
             return False
+        standard_id = regularize_SIA2_id(standard_id)
         return self.standard_id.split("#")[0] == standard_id.split("#")[0]
 
 
@@ -398,6 +472,9 @@ class RegistryResource(dalq.Record):
         "res_description",
         "reference_url",
         "creator_seq",
+        "created",
+        "updated",
+        "rights",
         "content_type",
         "source_format",
         "source_value",
@@ -410,21 +487,23 @@ class RegistryResource(dalq.Record):
         (f"\n  ivo_string_agg(COALESCE(intf_type, ''), '{TOKEN_SEP}')",
             "intf_types"),
         (f"\n  ivo_string_agg(COALESCE(intf_role, ''), '{TOKEN_SEP}')",
-            "intf_roles"), ]
+            "intf_roles"),
+        "alt_identifier"]
 
     def __init__(self, results, index, session=None):
-        dalq.Record.__init__(self, results, index, session)
+        dalq.Record.__init__(self, results, index, session=session)
 
         self._mapping["access_urls"
                       ] = self._parse_pseudo_array(self._mapping["access_urls"])
-        self._mapping["standard_ids"
-                      ] = self._parse_pseudo_array(self._mapping["standard_ids"])
+        self._mapping["standard_ids"] = [
+            regularize_SIA2_id(id) for id in
+                self._parse_pseudo_array(self._mapping["standard_ids"])]
         self._mapping["intf_types"
                       ] = self._parse_pseudo_array(self._mapping["intf_types"])
         self._mapping["intf_roles"
                       ] = self._parse_pseudo_array(self._mapping["intf_roles"])
 
-        self.interfaces = [Interface(*props)
+        self.interfaces = [Interface(props[0], standard_id=props[1], intf_type=props[2], intf_role=props[3])
                            for props in itertools.zip_longest(
             self["access_urls"],
             self["standard_ids"],
@@ -505,6 +584,14 @@ class RegistryResource(dalq.Record):
         """
         return self.get("reference_url", decode=True)
 
+    @property
+    def alt_identifier(self):
+        """Alternative identifier.
+
+        It is often used to provide the resource associated DOI.
+        """
+        return self.get("alt_identifier", decode=True)
+
     @property
     def creators(self):
         """
@@ -513,6 +600,25 @@ class RegistryResource(dalq.Record):
         """
         return self.get("creator_seq", default="", decode=True).split(";")
 
+    @property
+    def created(self):
+        """Date of creation of the resource."""
+        return self.get("created", decode=True)
+
+    @property
+    def updated(self):
+        """Date of last modification of the resource."""
+        return self.get("updated", decode=True)
+
+    @property
+    def rights(self):
+        """A statement of usage conditions for the content of the resource.
+
+        This information is often incomplete in the registry, you
+        might get more information at the ``reference_url``.
+        """
+        return self.get("rights", decode=True)
+
     @property
     def content_types(self):
         """
@@ -542,7 +648,11 @@ class RegistryResource(dalq.Record):
         by which a positional query against this resource should be "blurred"
         in order to get an appropriate match.
         """
-        return float(self.get("region_of_regard", 0))
+        # we get NULLs as NaNs here
+        val = self["region_of_regard"]
+        if numpy.isnan(val):
+            return None
+        return val
 
     @property
     def waveband(self):
@@ -609,18 +719,41 @@ class RegistryResource(dalq.Record):
                       std_only: bool = False):
         """returns a regtap.Interface class for service_type.
 
-        Parameters
-        ----------
-
         The meaning of the parameters is as for get_service.  This
         method does not return services, though, so you can use it to
         obtain access URLs and such for interfaces that pyVO does
-        not (directly) support. In addition,
+        not (directly) support.
+
+        Parameters
+        ----------
+
+        service_type : str
+            If you leave out ``service_type``, this will return a service
+            for "the" standard interface of the resource.  If a resource
+            has multiple standard capabilities (e.g., both TAP and SSAP
+            endpoints), this will raise a DALQueryError.
+
+            Otherwise, a service of the given service type will be returned.
+            Pass in an ivoid of a standard or one of the shorthands from
+            rtcons.SERVICE_TYPE_MAP, or "web" for a web page (the "service"
+            for this will be an object opening a web browser when you call
+            its query method).
+
+        lax : bool
+            If there are multiple capabilities for service_type, the
+            function choose the first matching capability by default
+            Pass lax=False to instead raise a DALQueryError.
 
         std_only : bool
             Only return interfaces declared as "std".  This is what you
             want when you want to construct pyVO service objects later.
             This parameter is ignored for the "web" service type.
+
+
+        Returns
+        -------
+
+        `~pyvo.registry.regtap.Interface`
         """
         if service_type == "web":
             # this works very much differently in the Registry
@@ -737,15 +870,16 @@ class RegistryResource(dalq.Record):
 
         Raises
         ------
-        RuntimeError
+        DALServiceError
            if the resource does not describe a searchable service.
         """
-        if not self.service:
+        try:
+            return self.service.search(*args, **keys)
+        except ValueError:
+            # I blindly assume the ValueError comes out of get_interface.
+            # But then that's likely enough.
             raise dalq.DALServiceError(
-                "resource, {}, is not a searchable service".format(
-                    self.short_name))
-
-        return self.service.search(*args, **keys)
+                f"Resource {self.ivoid} is not a searchable service")
 
     def describe(self, verbose=False, width=78, file=None):
         """
@@ -755,8 +889,10 @@ class RegistryResource(dalq.Record):
         ----------
         verbose : bool
             If false (default), only user-oriented information is
-            printed; if true, additional information will be printed
-            as well.
+            printed.
+            If true, additional information -- reference url,
+            reference to the related article, and alternative identifier
+            (often a DOI) -- will be printed if available.
         width : int
             Format the description with given character-width.
         out : writable file-like object
@@ -772,17 +908,13 @@ class RegistryResource(dalq.Record):
         if len(self._mapping["access_urls"]) == 1:
             print("Base URL: " + self.access_url, file=file)
         else:
-            print("Multi-capabilty service -- use get_service()", file=file)
+            print("Multi-capability service -- use get_service()", file=file)
 
         if self.res_description:
             print(file=file)
             print(para_format_desc(self.res_description), file=file)
             print(file=file)
 
-        if self.short_name:
-            print(
-                para_format_desc("Subjects: {}".format(self.short_name)),
-                file=file)
         if self.waveband:
             val = (str(v) for v in self.waveband)
             print(
@@ -790,6 +922,21 @@ class RegistryResource(dalq.Record):
                 file=file)
 
         if verbose:
+            if self.source_value:
+                print(f"\nSource: {self.source_value}", file=file)
+            if self.creators:
+                # if any creator has a name longer than 70 characters, we
+                # truncate it.
+                creators = [f"{creator[:70]}..." if len(creator) > 70
+                            else creator for creator in self.creators]
+                nmax_authors = 5
+                if len(creators) <= nmax_authors:
+                    print(f"Authors: {', '.join(creators)}", file=file)
+                else:
+                    print(f"Authors: {', '.join(creators[:nmax_authors])} et al.\n"
+                    "See creators attribute for the complete list of authors.", file=file)
+            if self.alt_identifier:
+                print(f"Alternative identifier: {self.alt_identifier}", file=file)
             if self.reference_url:
                 print("More info: " + self.reference_url, file=file)
 
@@ -839,10 +986,10 @@ class RegistryResource(dalq.Record):
 
     def _build_vosi_table(self, table_row, columns):
         """
-        return a io.vosi.vodataservice.Table element for a
+        return a io.vosi.vodataservice.VODataServiceTable element for a
         query result from get_tables.
         """
-        res = vodataservice.Table()
+        res = vodataservice.VODataServiceTable()
         res.name = table_row["table_name"]
         res.title = table_row["table_title"]
         res.description = table_row["table_description"]
@@ -858,7 +1005,7 @@ class RegistryResource(dalq.Record):
         """
         return the structure of the tables underlying the service.
 
-        This returns a dict with table names as keys and vosi.Table
+        This returns a dict with table names as keys and vodataservice.VODataServiceTable
         objects as values (pretty much what tables returns for a TAP
         service).  The table instances will have an ``origin`` attribute
         pointing back to the registry record.
@@ -899,12 +1046,15 @@ class RegistryResource(dalq.Record):
         return res
 
 
+@deprecated("1.5", "ivoid2service does not work in the presence of"
+    " multiple capabilities.  Use"
+    " registry.search(ivoid=...)[0].get_service('capname') instead.")
 def ivoid2service(ivoid, servicetype=None):
     """
     return service(s) for a given IVOID.
 
     The servicetype option specifies the kind of service requested
-    (conesearch, sia, ssa, slap, or tap).  By default, if none is
+    (conesearch, sia, sia2, ssa, slap, or tap).  By default, if none is
     given, a list of all matching services is returned.
 
     """
