@@ -24,7 +24,7 @@ from .import regtap
 
 __all__ = ["Freetext", "Author", "Servicetype", "Waveband",
            "Datamodel", "Ivoid", "UCD", "Spatial", "Spectral", "Temporal",
-           "Constraint", "build_regtap_query"]
+           "Constraint", "build_regtap_query", "RegTAPFeatureMissing"]
 
 
 # a mapping of service type shorthands to the ivoids of the
@@ -35,6 +35,8 @@ SERVICE_TYPE_MAP = dict((k, "ivo://ivoa.net/std/" + v)
                         for k, v in [
     ("image", "sia"),
     ("sia", "sia"),
+    # SIA2 is irregular
+    # funky scheme used by SIA2 without breaking everything else
     ("spectrum", "ssa"),
     ("ssap", "ssa"),
     ("ssa", "ssa"),
@@ -47,6 +49,20 @@ SERVICE_TYPE_MAP = dict((k, "ivo://ivoa.net/std/" + v)
 ])
 
 
+class RegTAPFeatureMissing(dalq.DALQueryError):
+    """
+    Raised when the current RegTAP server does not support a feature
+    needed for a constraint.
+
+    This could be that it is missing some ADQL feature indispensible
+    to write that constraint, or because it is missing a table or column.
+
+    To recover, choose another RegTAP service. Search constraining
+    ``datamodel="regtap"``, and then use `pyvo.registry.choose_RegTAP_service`
+    with a TAP access URL discovered in this way.
+    """
+
+
 class _AsIs(str):
     """a sentinel class make `make_sql_literal` not escape a string.
     """
@@ -154,10 +170,21 @@ class Constraint:
 
     takes_sequence = False
 
-    def get_search_condition(self):
+    def get_search_condition(self, service):
         """
         Formats this constraint to an ADQL fragment.
 
+        This takes the service the constraint is being executed on
+        as an argument because constraints may be written differently
+        depending on the service's features or refuse to run altogether.
+
+        Parameters
+        ----------
+        service : `dal.tap.TAPService`
+            The RegTAP service the query is supposed to be run on
+            (that is relevant because we adapt to the features available
+            on given services).
+
         Returns
         -------
         str
@@ -191,36 +218,71 @@ class Freetext(Constraint):
 
         Parameters
         ----------
-        *words: tuple of str
+        *words : tuple of str
             It is recommended to pass multiple words in multiple strings
             arguments.  You can pass in phrases (i.e., multiple words
             separated by space), but behaviour might then vary quite
             significantly between different registries.
         """
+        self.words = words
+
+    def get_search_condition(self, service):
         # cross-table ORs kill the query planner.  We therefore
         # write the constraint as an IN condition on a UNION
-        # of subqueries; it may look as if this has to be
-        # really slow, but in fact it's almost always a lot
-        # faster than direct ORs.
+        # of subqueries if we can (i.e., the service has UNION);
+        # It may look as if this has to be really slow, but in fact it's almost
+        # always a lot faster than direct ORs.
+        if service.get_tap_capability().get_adql().get_feature(
+                "ivo://ivoa.net/std/TAPRegExt#features-adql-sets", "UNION"):
+            return self._get_union_condition(service)
+        else:
+            self._extra_tables = ["rr.res_subject"]
+            return self._get_or_condition(service)
+
+    def _get_union_condition(self, service):
         base_queries = [
             "SELECT ivoid FROM rr.resource WHERE"
             " 1=ivo_hasword(res_description, {{{parname}}})",
             "SELECT ivoid FROM rr.resource WHERE"
             " 1=ivo_hasword(res_title, {{{parname}}})",
             "SELECT ivoid FROM rr.res_subject WHERE"
-            " res_subject ILIKE {{{parpatname}}}"]
+            " rr.res_subject.res_subject ILIKE {{{parpatname}}}"]
         self._fillers, subqueries = {}, []
 
-        for index, word in enumerate(words):
+        for index, word in enumerate(self.words):
             parname = "fulltext{}".format(index)
             parpatname = "fulltextpar{}".format(index)
             self._fillers[parname] = word
             self._fillers[parpatname] = '%' + word + '%'
-            for q in base_queries:
-                subqueries.append(q.format(**locals()))
+            args = locals()
+            subqueries.append(" UNION ".join(
+                q.format(**args) for q in base_queries))
 
-        self._condition = "ivoid IN ({})".format(
-            " UNION ".join(subqueries))
+        self._condition = " AND ".join(
+            f"ivoid IN ({part})" for part in subqueries)
+
+        return super().get_search_condition(service)
+
+    def _get_or_condition(self, service):
+        base_queries = [
+            " 1=ivo_hasword(res_description, {{{parname}}})",
+            " 1=ivo_hasword(res_title, {{{parname}}})",
+            " rr.res_subject.res_subject ILIKE {{{parpatname}}}"]
+        self._fillers, conditions = {}, []
+
+        for index, word in enumerate(self.words):
+            parname = "fulltext{}".format(index)
+            parpatname = "fulltextpar{}".format(index)
+            self._fillers[parname] = word
+            self._fillers[parpatname] = '%' + word + '%'
+            args = locals()
+            conditions.append(" OR ".join(
+                q.format(**args) for q in base_queries))
+
+        self._condition = " AND ".join(f"({part})"
+            for part in conditions)
+
+        return super().get_search_condition(service)
 
 
 class Author(Constraint):
@@ -237,7 +299,7 @@ class Author(Constraint):
 
         Parameters
         ----------
-        name: str
+        name : str
             Note that regrettably there are no guarantees as to how authors
             are written in the VO.  This means that you will generally have
             to write things like ``%Hubble%`` (% being “zero or more
@@ -257,6 +319,7 @@ class Servicetype(Constraint):
     * ``image`` (image services; at this point equivalent to sia, but
       scheduled to include sia2, too)
     * ``sia`` (SIAP version 1 services)
+    * ``sia2`` (SIAP version 2 services)
     * ``spectrum``, ``ssa``, ``ssap`` (all synonymous for spectral
       services, prefer ``spectrum``)
     * ``scs``, ``conesearch`` (synonymous for cone search services, prefer
@@ -266,10 +329,10 @@ class Servicetype(Constraint):
 
     You can also pass in the standards' ivoid (which
     generally looks like
-    ``ivo://ivoa.net/std/<standardname>`` and have to be URIs with
-    a scheme part in any case); note, however, that for standards
-    pyVO does not know about it will not build service instances for
-    you.
+    ``ivo://ivoa.net/std/<standardname>`` (except for SIA2) and have
+    to be URIs with a scheme part in any case); note, however, that for
+    standards pyVO does not know about it will not build service instances
+    for you.
 
     Multiple service types can be passed in; a match in that case
     is for records having any of the service types passed in.
@@ -290,27 +353,44 @@ class Servicetype(Constraint):
 
         Parameters
         ----------
-        *stds: tuple of str
+        *stds : tuple of str
             one or more standards identifiers.  The constraint will
             match records that have any of them.
         """
         self.stdids = set()
+        self.extra_fragments = []
 
         for std in stds:
             if std in SERVICE_TYPE_MAP:
                 self.stdids.add(SERVICE_TYPE_MAP[std])
             elif "://" in std:
                 self.stdids.add(std)
+            elif std == 'sia2':
+                self.extra_fragments.append(
+                    "standard_id like 'ivo://ivoa.net/std/sia#query-2.%'")
             else:
                 raise dalq.DALQueryError("Service type {} is neither a full"
                                          " standard URI nor one of the bespoke identifiers"
-                                         " {}".format(std, ", ".join(SERVICE_TYPE_MAP)))
+                                         " {}, sia2".format(std, ", ".join(SERVICE_TYPE_MAP)))
 
-    def get_search_condition(self):
+    def clone(self):
+        """returns a copy of this servicetype constraint.
+        """
+        new_constraint = Servicetype()
+        new_constraint.stdids = set(self.stdids)
+        new_constraint.extra_fragments = self.extra_fragments[:]
+        return new_constraint
+
+    def get_search_condition(self, service):
         # we sort the stdids to make it easy for tests (and it's
         # virtually free for the small sets we have here).
-        return "standard_id IN ({})".format(
-            ", ".join(make_sql_literal(s) for s in sorted(self.stdids)))
+        fragments = []
+        std_ids = ", ".join(make_sql_literal(s)
+            for s in sorted(self.stdids))
+        if std_ids:
+            fragments.append(f"standard_id IN ({std_ids})")
+
+        return " OR ".join(fragments + self.extra_fragments)
 
     def include_auxiliary_services(self):
         """returns a Servicetype constraint that has self's
@@ -318,8 +398,13 @@ class Servicetype(Constraint):
 
         This is a convenience to maintain registry.search's signature.
         """
-        return Servicetype(*(self.stdids | set(
-            std + '#aux' for std in self.stdids)))
+        expanded = self.clone()
+        expanded.stdids |= set(
+            std + '#aux' for std in expanded.stdids)
+        if "standard_id like 'ivo://ivoa.net/std/sia#query-2.%'" in expanded.extra_fragments:
+            expanded.extra_fragments.append(
+                "standard_id like 'ivo://ivoa.net/std/sia#query-aux-2.%'")
+        return expanded
 
 
 class Waveband(Constraint):
@@ -343,7 +428,7 @@ class Waveband(Constraint):
 
         Parameters
         ----------
-        *bands: tuple of strings
+        *bands : tuple of strings
             One or more of the terms given in http://www.ivoa.net/rdf/messenger.
             The constraint matches when a resource declares at least
             one of the messengers listed.
@@ -503,6 +588,14 @@ class Spatial(Constraint):
 
     .. _MOC: https://www.ivoa.net/documents/MOC/
 
+    To find resources which coverage is enclosed in a region,
+
+        >>> enclosed = registry.Spatial("0/0-11", intersect="enclosed")
+
+    To find resources which coverage intersects a region,
+
+        >>> overlaps = registry.Spatial("0/0-11", intersect="overlaps")
+
     When you already have an astropy SkyCoord::
 
         >>> from astropy.coordinates import SkyCoord
@@ -514,12 +607,11 @@ class Spatial(Constraint):
         >>> resources = registry.Spatial((SkyCoord("23d +3d"), 3))
     """
     _keyword = "spatial"
-    _condition = "1 = CONTAINS({geom}, coverage)"
     _extra_tables = ["rr.stc_spatial"]
 
     takes_sequence = True
 
-    def __init__(self, geom_spec, order=6):
+    def __init__(self, geom_spec, order=6, intersect="covers"):
         """
 
         Parameters
@@ -530,14 +622,21 @@ class Spatial(Constraint):
             as a DALI polygon.  Additionally, strings are interpreted
             as ASCII MOCs, SkyCoords as points, and a pair of a
             SkyCoord and a float as a circle.  Other types (proper
-            geometries or pymoc objects) might be supported in the
+            geometries or MOCPy objects) might be supported in the
             future.
         order : int, optional
             Non-MOC geometries are converted to MOCs before comparing
-            them to the resource coverage.  By default, this contrains
+            them to the resource coverage.  By default, this constraint
             uses order 6, which corresponds to about a degree of resolution
             and is what RegTAP recommends as a sane default for the
             order actually used for the coverages in the database.
+        intersect : str, optional
+            Allows to specify the connection between the resource coverage
+            and the *geom_spec*. The possible values are 'covers' for services
+            that completely cover the *geom_spec* region, 'enclosed' for services
+            completely enclosed in the region and 'overlaps' for services which
+            coverage intersect the region.
+
         """
         def tomoc(s):
             return _AsIs("MOC({}, {})".format(order, s))
@@ -567,7 +666,33 @@ class Spatial(Constraint):
         else:
             raise ValueError("This constraint needs DALI-style geometries.")
 
-        self._fillers = {"geom": geom}
+        if intersect == "covers":
+            self._condition = f"1 = CONTAINS({geom}, coverage)"
+        elif intersect == "enclosed":
+            self._condition = f"1 = CONTAINS(coverage, {geom})"
+        elif intersect == "overlaps":
+            self._condition = f"1 = INTERSECTS(coverage, {geom})"
+        else:
+            raise ValueError("'intersect' should be one of 'covers', 'enclosed', or 'overlaps' "
+                             f"but its current value is '{intersect}'.")
+
+    def get_search_condition(self, service):
+        # we *could* make this a bit less demanding on the server
+        # if we MOC-ified the geometries locally -- but then we'd
+        # have to depend on pymoc, and that's too high a price for
+        # something as esoteric as a server that understands
+        # MOC-based geometries but does not have a MOC function.
+        if not service.get_tap_capability().get_adql().get_feature(
+                "ivo://org.gavo.dc/std/exts#extra-adql-keywords", "MOC"):
+            raise RegTAPFeatureMissing("Current RegTAP service does not support MOC.")
+
+        # We should compare case-insensitively here, but then we don't
+        # with delimited identifiers -- in the end, that would have to
+        # be handled in dal.vosi.VOSITables.
+        if "rr.stc_spatial" not in service.tables:
+            raise RegTAPFeatureMissing("stc_spatial missing on current RegTAP service")
+
+        return super().get_search_condition(service)
 
 
 class Spectral(Constraint):
@@ -659,6 +784,12 @@ class Spectral(Constraint):
 
         raise ValueError(f"Cannot make a spectral quantity out of {quant}")
 
+    def get_search_condition(self, service):
+        if "rr.stc_spectral" not in service.tables:
+            raise RegTAPFeatureMissing("stc_spectral missing on current RegTAP service")
+
+        return super().get_search_condition(service)
+
 
 class Temporal(Constraint):
     """
@@ -731,22 +862,34 @@ class Temporal(Constraint):
                              " single time instants.")
         return val
 
+    def get_search_condition(self, service):
+        if "rr.stc_temporal" not in service.tables:
+            raise RegTAPFeatureMissing("stc_temporal missing on current RegTAP service")
+
+        return super().get_search_condition(service)
+
 
 # NOTE: If you add new Contraint-s, don't forget to add them in
-# registry.__init__ and in docs/registry/index.rst.
+# registry.__init__, in docs/registry/index.rst and in the docstring
+# of regtap.query.
 
 
-def build_regtap_query(constraints):
+def build_regtap_query(constraints, service):
     """returns a RegTAP query ready for submission from a list of
     Constraint instances.
 
     Parameters
     ----------
-    constraints: sequence of `Constraint`-s
+    constraints : sequence of ``Constraint``-s
         A sequence of constraints for a RegTAP query.  All of them
         will become part of a conjunction (i.e., all of them have
         to be satisfied for a record to match).
 
+    service : `~pyvo.dal.tap.TAPService`
+        The RegTAP service the query is supposed to be run on
+        (that is relevant because we adapt to the features available
+        on given services).
+
     Returns
     -------
     str
@@ -761,10 +904,12 @@ def build_regtap_query(constraints):
         if isinstance(constraint, str):
             constraint = Freetext(constraint)
 
-        serialized.append("(" + constraint.get_search_condition() + ")")
+        serialized.append(
+            "(" + constraint.get_search_condition(service) + ")")
         extra_tables |= set(constraint._extra_tables)
 
-    joined_tables = ["rr.resource", "rr.capability", "rr.interface"
+    joined_tables = ["rr.resource", "rr.capability", "rr.interface",
+                     "rr.alt_identifier"
                      ] + list(extra_tables)
 
     # see comment in regtap.RegistryResource for the following
