@@ -11,17 +11,19 @@ from urllib.parse import parse_qsl
 
 import pytest
 
+from astropy import time
+
 from pyvo.registry import regtap
 from pyvo.registry import rtcons
 from pyvo.registry.regtap import REGISTRY_BASEURL
 from pyvo.registry import search as regsearch
 from pyvo.dal import DALOverflowWarning
 from pyvo.dal import query as dalq
-from pyvo.dal import tap
+from pyvo.dal import tap, sia2
 
 from astropy.utils.data import get_pkg_data_contents
 
-from .commonfixtures import messenger_vocabulary  # noqa: F401
+from .commonfixtures import messenger_vocabulary, FAKE_GAVO  # noqa: F401
 
 
 get_pkg_data_contents = partial(
@@ -67,7 +69,7 @@ def keywords_fixture(mocker):
         assert "ivo_hasword(res_description, 'vizier')" in query
         assert "1=ivo_hasword(res_title, 'vizier')" in query
 
-        assert " res_subject ILIKE '%pulsar%'" in query
+        assert ".res_subject ILIKE '%pulsar%'" in query
         assert "1=ivo_hasword(res_description, 'pulsar')" in query
         assert "1=ivo_hasword(res_title, 'pulsar')" in query
 
@@ -86,8 +88,8 @@ def single_keyword_fixture(mocker):
         data = dict(parse_qsl(request.body))
         query = data['QUERY']
 
-        assert "WHERE res_subject ILIKE '%single%'" in query
-        assert "WHERE 1=ivo_hasword(res_description, 'single') UNION" in query
+        assert "OR  rr.res_subject.res_subject ILIKE '%single%'" in query
+        assert "1=ivo_hasword(res_description, 'single') " in query
         assert "1=ivo_hasword(res_title, 'single')" in query
 
         return get_pkg_data_contents('data/regtap.xml')
@@ -211,18 +213,18 @@ class TestInterfaceClass:
         assert not intf.is_vosi
 
     def test_repr(self):
-        intf = regtap.Interface("http://example.org", "ivo://gavo/std/a",
-                                "vs:paramhttp", "std")
+        intf = regtap.Interface("http://example.org", standard_id="ivo://gavo/std/a",
+                                intf_type="vs:paramhttp", intf_role="std")
         assert (repr(intf) == "Interface('http://example.org',"
-                " 'ivo://gavo/std/a', 'vs:paramhttp', 'std')")
-        intf = regtap.Interface("http://example.org", "ivo://gavo/std/a",
-                                None, None)
+                " standard_id='ivo://gavo/std/a', intf_type='vs:paramhttp', intf_role='std')")
+        intf = regtap.Interface("http://example.org", standard_id="ivo://gavo/std/a",
+                                intf_type=None, intf_role=None)
         assert repr(intf) == ("Interface('http://example.org',"
-                              " 'ivo://gavo/std/a', None, None)")
+                              " standard_id='ivo://gavo/std/a', intf_type=None, intf_role=None)")
 
     def test_unknown_standard(self):
-        intf = regtap.Interface("http://example.org", "ivo://gavo/std/a",
-                                "vs:paramhttp", "std")
+        intf = regtap.Interface("http://example.org", standard_id="ivo://gavo/std/a",
+                                intf_type="vs:paramhttp", intf_role="std")
         assert intf.is_standard
         with pytest.raises(ValueError) as excinfo:
             intf.to_service()
@@ -232,15 +234,20 @@ class TestInterfaceClass:
             " id ivo://gavo/std/a.")
 
     def test_known_standard(self):
-        intf = regtap.Interface("http://example.org",
-                                "ivo://ivoa.net/std/tap#aux", "vs:paramhttp", "std")
+        intf = regtap.Interface("http://example.org", standard_id="ivo://ivoa.net/std/tap#aux",
+                                intf_type="vs:paramhttp", intf_role="std")
         assert isinstance(intf.to_service(), tap.TAPService)
         assert not intf.is_vosi
 
+    def test_sia2_standard(self):
+        intf = regtap.Interface("http://example.org", standard_id="ivo://ivoa.net/std/sia2",
+                                intf_type="vs:paramhttp", intf_role="std")
+        assert isinstance(intf.to_service(), sia2.SIA2Service)
+        assert not intf.is_vosi
+
     def test_secondary_interface(self):
-        intf = regtap.Interface("http://example.org",
-                                "ivo://ivoa.net/std/tap#aux",
-                                "vs:webbrowser", "web")
+        intf = regtap.Interface("http://example.org", standard_id="ivo://ivoa.net/std/tap#aux",
+                                intf_type="vs:webbrowser", intf_role="web")
 
         with pytest.raises(ValueError) as excinfo:
             intf.to_service()
@@ -249,9 +256,8 @@ class TestInterfaceClass:
             "This is not a standard interface.  PyVO cannot speak to it.")
 
     def test_VOSI(self):
-        intf = regtap.Interface("http://example.org",
-                                "ivo://ivoa.net/std/vosi#capabilities",
-                                "vs:ParamHTTP", "std")
+        intf = regtap.Interface("http://example.org", standard_id="ivo://ivoa.net/std/vosi#capabilities",
+                                intf_type="vs:ParamHTTP", intf_role="std")
         assert intf.is_vosi
 
 
@@ -340,13 +346,13 @@ def get_regtap_results(**kwargs):
 
 def test_spatial():
     assert (rtcons.keywords_to_constraints({
-            "spatial": (23, -40)})[0].get_search_condition()
+            "spatial": (23, -40)})[0].get_search_condition(FAKE_GAVO)
             == "1 = CONTAINS(MOC(6, POINT(23, -40)), coverage)")
 
 
 def test_spectral():
     assert (rtcons.keywords_to_constraints({
-            "spectral": (1e-17, 2e-17)})[0].get_search_condition()
+            "spectral": (1e-17, 2e-17)})[0].get_search_condition(FAKE_GAVO)
             == "1 = ivo_interval_overlaps(spectral_start, spectral_end, 1e-17, 2e-17)")
 
 
@@ -360,7 +366,7 @@ def test_to_table(multi_interface_fixture, capabilities):
     assert (t["description"][0][:40]
             == 'Spectra from the Flash and Heros Echelle')
     assert (t["interfaces"][0]
-            == 'datalink#links-1.0, soda#sync-1.0, ssa, tap#aux, web')
+            == 'datalink#links-1.1, soda#sync-1.0, ssa, tap#aux, web')
 
 
 @pytest.fixture(name='rt_pulsar_distance')
@@ -376,12 +382,18 @@ def test_record_fields(rt_pulsar_distance):
     assert rec.res_title == "Catalog of 558 Pulsars"
     assert rec.content_levels == ['research']
     assert rec.res_description[:20] == "The catalogue is an up-to-date"[:20]
-    assert rec.reference_url == "http://cdsarc.unistra.fr/cgi-bin/cat/VII/156"
+    assert rec.reference_url == "https://cdsarc.cds.unistra.fr/viz-bin/cat/VII/156"
     assert rec.creators == ['Taylor J.H.', ' Manchester R.N.', ' Lyne A.G.']
     assert rec.content_types == ['catalog']
     assert rec.source_format == "bibcode"
     assert rec.source_value == "1993ApJS...88..529T"
+    assert rec.region_of_regard is None
     assert rec.waveband == ['radio']
+    assert rec.created == "1999-03-02T10:21:53"
+    # updated might break when regenerating data/regtap.xml,
+    # replace by the new date
+    assert rec.updated == "2021-10-21T00:00:00"
+    assert rec.rights == "https://cds.unistra.fr/vizier-org/licences_vizier.html"
     # access URL, standard_id and friends exercised in TestInterfaceSelection
 
 
@@ -403,7 +415,7 @@ class TestResultIndexing:
         with pytest.raises(IndexError) as excinfo:
             rt_pulsar_distance[40320]
         assert (str(excinfo.value)
-                == "index 40320 is out of bounds for axis 0 with size 23")
+                == f"index 40320 is out of bounds for axis 0 with size {len(rt_pulsar_distance)}")
 
     def test_bad_key(self, rt_pulsar_distance):
         with pytest.raises(KeyError) as excinfo:
@@ -432,7 +444,7 @@ class TestInterfaceSelection:
 
     def test_access_modes(self, flash_service):
         assert set(flash_service.access_modes()) == {
-            'datalink#links-1.0', 'soda#sync-1.0', 'ssa', 'tap#aux',
+            'datalink#links-1.1', 'soda#sync-1.0', 'ssa', 'tap#aux',
             'web'}
 
     def test_standard_id_multi(self, flash_service):
@@ -440,7 +452,7 @@ class TestInterfaceSelection:
             _ = flash_service.standard_id
 
         assert str(excinfo.value) == ("This resource supports several"
-                                      " standards (datalink#links-1.0, soda#sync-1.0, ssa,"
+                                      " standards (datalink#links-1.1, soda#sync-1.0, ssa,"
                                       " tap#aux, web).  Use get_service or restrict your query"
                                       " using Servicetype.")
 
@@ -451,6 +463,17 @@ class TestInterfaceSelection:
         assert (svc.access_url
                 == "http://dc.zah.uni-heidelberg.de/flashheros/q/web/form")
 
+        import webbrowser
+        orig_open = webbrowser.open
+        try:
+            open_args = []
+            webbrowser.open = lambda *args: open_args.append(args)
+            svc.search()
+            assert open_args == [
+                ("http://dc.zah.uni-heidelberg.de/flashheros/q/web/form", 2)]
+        finally:
+            webbrowser.open = orig_open
+
     def test_get_aux_interface(self, flash_service):
         svc = flash_service.get_service("tap#aux")
         assert (svc._baseurl
@@ -514,6 +537,46 @@ class TestInterfaceSelection:
             resource.get_service('tap')
         assert (str(excinfo.value) == "No matching interface.")
 
+    def test_sia2_query(self):
+        rec = _makeRegistryRecord(
+            access_urls=["http://sia2.example.com", "http://sia.example.com"],
+            standard_ids=[
+                "ivo://ivoa.net/std/sia#query-2.0",
+                "ivo://ivoa.net/std/sia"],
+            intf_roles=["std"] * 2,
+            intf_types=["vs:paramhttp"] * 2)
+        assert rec.access_modes() == {"sia", "sia2"}
+        assert rec.get_interface("sia2").access_url == 'http://sia2.example.com'
+        assert rec.get_interface("sia").access_url == 'http://sia.example.com'
+
+    def test_sia2_aux(self):
+        rec = _makeRegistryRecord(
+            access_urls=["http://sia2.example.com", "http://sia.example.com"],
+            standard_ids=[
+                "ivo://ivoa.net/std/sia#query-aux-2.0",
+                "ivo://ivoa.net/std/sia#aux"],
+            intf_roles=["std"] * 2,
+            intf_types=["vs:paramhttp"] * 2)
+        assert rec.access_modes() == {"sia#aux", "sia2#aux"}
+        assert rec.get_interface("sia2").access_url == 'http://sia2.example.com'
+        assert rec.get_interface("sia").access_url == 'http://sia.example.com'
+
+    def test_non_standard_interface(self):
+        intf = regtap.Interface("http://url", standard_id="", intf_type="", intf_role="")
+        assert intf.supports("ivo://ivoa.net/std/sia") is False
+
+    def test_supports_none(self):
+        intf = regtap.Interface("http://url", standard_id="", intf_type="", intf_role="")
+        assert intf.supports(None) is False
+
+    def test_non_searchable_service(self):
+        rec = _makeRegistryRecord()
+        with pytest.raises(dalq.DALServiceError) as excinfo:
+            rec.search()
+
+        assert str(excinfo.value) == (
+            "Resource ivo://pyvo/test_regtap.py is not a searchable service")
+
 
 class _FakeResult:
     """A fake class just sufficient for giving dal.query.Record enough
@@ -536,7 +599,7 @@ class _FakeResult:
         self.resultstable = _
 
 
-def _makeRegistryRecord(overrides):
+def _makeRegistryRecord(**overrides):
     """returns a minimal RegistryResource instance, overriding
     some built-in defaults with the dict overrides.
     """
@@ -557,12 +620,11 @@ class TestInterfaceRejection:
     """
 
     def test_nonunique(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a", "http://b"],
-            "standard_ids": ["ivo://ivoa.net/std/tap"] * 2,
-            "intf_types": ["vs:paramhttp"] * 2,
-            "intf_roles": ["std"] * 2,
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a", "http://b"],
+            standard_ids=["ivo://ivoa.net/std/tap"] * 2,
+            intf_types=["vs:paramhttp"] * 2,
+            intf_roles=["std"] * 2)
         with pytest.raises(ValueError) as excinfo:
             rsc.get_service("tap", lax=False)
 
@@ -572,39 +634,38 @@ class TestInterfaceRejection:
             " registry.search?  Or use lax=True?")
 
     def test_nonunique_lax(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a", "http://b"],
-            "standard_ids": ["ivo://ivoa.net/std/tap"] * 2,
-            "intf_types": ["vs:paramhttp"] * 2,
-            "intf_roles": ["std"] * 2,
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a", "http://b"],
+            standard_ids=["ivo://ivoa.net/std/tap"] * 2,
+            intf_types=["vs:paramhttp"] * 2,
+            intf_roles=["std"] * 2)
 
         assert (rsc.get_service("tap")._baseurl
                 == "http://a")
 
     def test_nonstd_ignored(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a", "http://b"],
-            "standard_ids": ["ivo://ivoa.net/std/tap"] * 2,
-            "intf_types": ["vs:paramhttp"] * 2,
-            "intf_roles": ["std", ""]
-        })
-
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a", "http://b"],
+            standard_ids=["ivo://ivoa.net/std/tap"] * 2,
+            intf_types=["vs:paramhttp"] * 2,
+            intf_roles=["std", ""])
         assert (rsc.get_service("tap", lax=False)._baseurl
                 == "http://a")
 
     def test_select_single_matching_service(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a", "http://b"],
-            "standard_ids": ["", "ivo://ivoa.net/std/tap"],
-            "intf_types": ["vs:webbrowser", "vs:paramhttp"],
-            "intf_roles": ["", "std"]
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a", "http://b"],
+            standard_ids=["", "ivo://ivoa.net/std/tap"],
+            intf_types=["vs:webbrowser", "vs:paramhttp"],
+            intf_roles=["", "std"])
 
         assert (rsc.service._baseurl == "http://b")
+        # this makes sure caching the service obtained doesn't break
+        # things
+        assert (rsc.service._baseurl == "http://b")
 
     def test_capless(self):
-        rsc = _makeRegistryRecord({})
+        rsc = _makeRegistryRecord()
 
         with pytest.raises(ValueError) as excinfo:
             rsc.service._baseurl
@@ -624,7 +685,7 @@ def test_maxrec():
 @pytest.mark.remote_data
 def test_get_contact():
     rsc = _makeRegistryRecord(
-        {"ivoid": "ivo://org.gavo.dc/flashheros/q/ssa"})
+        ivoid="ivo://org.gavo.dc/flashheros/q/ssa")
     assert (rsc.get_contact()
             == "GAVO Data Center Team (++49 6221 54 1837)"
             " <gavo@ari.uni-heidelberg.de>")
@@ -657,12 +718,11 @@ class TestExtraResourceMethods:
     """
 
     def test_unique_standard_id(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a"],
-            "standard_ids": ["ivo://ivoa.net/std/tap"],
-            "intf_types": ["vs:paramhttp"],
-            "intf_roles": ["std"]
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a"],
+            standard_ids=["ivo://ivoa.net/std/tap"],
+            intf_types=["vs:paramhttp"],
+            intf_roles=["std"])
         assert rsc.standard_id == "ivo://ivoa.net/std/tap"
 
     def test_describe_multi(self, flash_service):
@@ -671,19 +731,54 @@ class TestExtraResourceMethods:
         output = out.getvalue()
 
         assert "Flash/Heros SSAP" in output
-        assert ("Access modes: datalink#links-1.0, soda#sync-1.0,"
+        assert ("Access modes: datalink#links-1.1, soda#sync-1.0,"
                 " ssa, tap#aux, web" in output)
-        assert "Multi-capabilty service" in output
-
+        assert "Multi-capability service" in output
+        assert "Source: 1996A&A...312..539S" in output
+        assert "Authors: Wolf" in output
+        assert "Alternative identifier: doi:10.21938/" in output
         assert "More info: http://dc.zah" in output
 
+    def test_describe_long_authors_list(self):
+        """Check that long list of authors use et al.."""
+        rsc = _makeRegistryRecord(
+            access_urls=[],
+            standard_ids=["ivo://pyvo/test"],
+            short_name=["name"],
+            intf_types=[],
+            intf_roles=[],
+            creator_seq=["a;" * 6],
+            res_title=["title"]
+        )
+        out = io.StringIO()
+        rsc.describe(verbose=True, file=out)
+        output = out.getvalue()
+        # output should cut at 5 authors
+        assert "Authors: a, a, a, a, a et al." in output
+
+    def test_describe_long_author_name(self):
+        """Check that long author names are truncated."""
+        rsc = _makeRegistryRecord(
+            access_urls=[],
+            standard_ids=["ivo://pyvo/test"],
+            short_name=["name"],
+            intf_types=[],
+            intf_roles=[],
+            creator_seq=["a" * 71],
+            res_title=["title"]
+        )
+        out = io.StringIO()
+        rsc.describe(verbose=True, file=out)
+        output = out.getvalue()
+        # should cut the long author name at 70 characters
+        assert f"Authors: {'a'*70}..." in output
+
     def test_no_access_url(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": [],
-            "standard_ids": [],
-            "intf_types": [],
-            "intf_roles": []
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=[],
+            standard_ids=[],
+            intf_types=[],
+            intf_roles=[])
         with pytest.raises(dalq.DALQueryError) as excinfo:
             rsc.access_url
 
@@ -691,21 +786,19 @@ class TestExtraResourceMethods:
                                       " ivo://pyvo/test_regtap.py has no queriable interfaces.")
 
     def test_unique_access_url(self):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a"],
-            "standard_ids": ["ivo://ivoa.net/std/tap"],
-            "intf_types": ["vs:paramhttp"],
-            "intf_roles": [""]
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a"],
+            standard_ids=["ivo://ivoa.net/std/tap"],
+            intf_types=["vs:paramhttp"],
+            intf_roles=[""])
         assert rsc.access_url == "http://a"
 
     def test_ambiguous_access_url_warns(self, recwarn):
-        rsc = _makeRegistryRecord({
-            "access_urls": ["http://a", "http://b"],
-            "standard_ids": ["ivo://ivoa.net/std/tap"] * 2,
-            "intf_types": ["vs:paramhttp"] * 2,
-            "intf_roles": ["std"] * 2,
-        })
+        rsc = _makeRegistryRecord(
+            access_urls=["http://a", "http://b"],
+            standard_ids=["ivo://ivoa.net/std/tap"] * 2,
+            intf_types=["vs:paramhttp"] * 2,
+            intf_roles=["std"] * 2)
         assert rsc.access_url == "http://a"
         assert ('The resource ivo://pyvo/test_regtap.py has multipl' in
                 [str(w.message)[:50] for w in recwarn])
@@ -714,11 +807,10 @@ class TestExtraResourceMethods:
 # TODO: While I suppose the contact test should keep requiring network,
 # I think we should can the network responses involved in the following;
 # the stuff might change upstream any time and then break our unit tests.
-@pytest.mark.remote_data
 @pytest.fixture(name='flash_tables')
 def _flash_tables():
     rsc = _makeRegistryRecord(
-        {"ivoid": "ivo://org.gavo.dc/flashheros/q/ssa"})
+        ivoid="ivo://org.gavo.dc/flashheros/q/ssa")
     return rsc.get_tables()
 
 
@@ -727,7 +819,7 @@ class TestGetTables:
     @pytest.mark.remote_data
     def test_get_tables_limit_enforced(self):
         rsc = _makeRegistryRecord(
-            {"ivoid": "ivo://org.gavo.dc/tap"})
+            ivoid="ivo://org.gavo.dc/tap")
         with pytest.raises(dalq.DALQueryError) as excinfo:
             rsc.get_tables()
 
@@ -776,3 +868,31 @@ class TestGetTables:
 
         assert (getflashcol("ssa_fluxcalib").description
                 == "Type of flux calibration")
+
+
+@pytest.mark.remote_data
+def test_sia2_service_operation():
+    svcs = regsearch(
+        servicetype='sia2',
+        ivoid='ivo://cadc.nrc.ca/sia')
+    assert len(svcs) == 1
+
+    res = svcs[0].search(pos=(30, 40, 0.1),
+        time=(time.Time(58794.9, format="mjd"),
+            time.Time(58795, format="mjd")))
+    assert len(res) > 10
+    assert "s_dec" in res.to_table().columns
+
+
+@pytest.mark.remote_data
+def test_endpoint_switching():
+    alt_svc = "http://vao.stsci.edu/RegTAP/TapService.aspx"
+    previous_url = regtap.REGISTRY_BASEURL
+    try:
+        regtap.choose_RegTAP_service(alt_svc)
+        assert regtap.get_RegTAP_service()._baseurl == alt_svc
+
+        res = regtap.search(keywords="wirr")
+        assert len(res) > 0
+    finally:
+        regtap.choose_RegTAP_service(previous_url)
